export class ApiGraph {
    packages = new Map();
    endpoints = new Map();
    types = new Map();
    errors = new Map();
    constructor(initialDefinition) {
        for (const package_ of Object.values(initialDefinition.packages)) {
            this.processApiPackage(package_, { api: initialDefinition, parent: undefined });
        }
    }
    processApiPackage(apiPackage, { api, parent }) {
        this.createPackage(apiPackage.packageId, { parent });
        for (const subPackageId of apiPackage.packages) {
            const apiSubPackage = api.packages[subPackageId];
            if (apiSubPackage == null) {
                throw new Error("Package does not exist: " + subPackageId);
            }
            this.processApiPackage(apiSubPackage, { api, parent: apiPackage.packageId });
        }
        for (const endpointId of apiPackage.endpoints) {
            this.createEndpoint(endpointId, { parent: apiPackage.packageId });
        }
        for (const typeId of apiPackage.types) {
            this.createType(typeId, { parent: apiPackage.packageId });
        }
        for (const errorId of apiPackage.errors) {
            this.createError(errorId, { parent: apiPackage.packageId });
        }
    }
    getPackageParent(packageId) {
        const package_ = this.getPackageOrThrow(packageId);
        return package_.parent;
    }
    getEndpointParent(endpointId) {
        const endpoint = this.getEndpointOrThrow(endpointId);
        return endpoint.parent;
    }
    getTypeParent(typeId) {
        const type = this.getTypeOrThrow(typeId);
        return type.parent;
    }
    getErrorParent(errorId) {
        const error = this.getErrorOrThrow(errorId);
        return error.parent;
    }
    createPackage(packageId, { parent }) {
        this.packages.set(packageId, {
            packageId,
            parent,
            packages: new Set(),
            endpoints: new Set(),
            types: new Set(),
            errors: new Set(),
        });
    }
    deletePackage(packageId) {
        const package_ = this.getPackageOrThrow(packageId);
        for (const subPackageId of package_.packages) {
            this.deletePackage(subPackageId);
        }
        for (const endpointId of package_.endpoints) {
            this.deleteEndpoint(endpointId);
        }
        for (const typeId of package_.types) {
            this.deleteType(typeId);
        }
        for (const errorId of package_.errors) {
            this.deleteError(errorId);
        }
        this.packages.delete(packageId);
        if (package_.parent != null) {
            const parent = this.getPackageOrThrow(package_.parent);
            parent.packages.delete(packageId);
        }
    }
    createEndpoint(endpointId, { parent }) {
        const package_ = this.getPackageOrThrow(parent);
        package_.endpoints.add(endpointId);
        this.endpoints.set(endpointId, {
            endpointId,
            parent,
        });
    }
    deleteEndpoint(endpointId) {
        const endpoint = this.getEndpointOrThrow(endpointId);
        this.endpoints.delete(endpointId);
        const package_ = this.getPackageOrThrow(endpoint.parent);
        package_.endpoints.delete(endpointId);
    }
    createType(typeId, { parent }) {
        const package_ = this.getPackageOrThrow(parent);
        package_.types.add(typeId);
        this.types.set(typeId, {
            typeId,
            parent,
        });
    }
    deleteType(typeId) {
        const type = this.getTypeOrThrow(typeId);
        this.types.delete(typeId);
        const package_ = this.getPackageOrThrow(type.parent);
        package_.types.delete(typeId);
    }
    createError(errorId, { parent }) {
        const package_ = this.getPackageOrThrow(parent);
        package_.errors.add(errorId);
        this.errors.set(errorId, {
            errorId,
            parent,
        });
    }
    deleteError(errorId) {
        const error = this.getErrorOrThrow(errorId);
        this.errors.delete(errorId);
        const package_ = this.getPackageOrThrow(error.parent);
        package_.errors.delete(errorId);
    }
    getPackageOrThrow(packageId) {
        const package_ = this.packages.get(packageId);
        if (package_ == null) {
            throw new Error(`Package ${packageId} does not exist`);
        }
        return package_;
    }
    getEndpointOrThrow(endpointId) {
        const endpoint = this.endpoints.get(endpointId);
        if (endpoint == null) {
            throw new Error(`Endpoint ${endpoint} does not exist`);
        }
        return endpoint;
    }
    getTypeOrThrow(typeId) {
        const type = this.types.get(typeId);
        if (type == null) {
            throw new Error(`Type ${type} does not exist`);
        }
        return type;
    }
    getErrorOrThrow(errorId) {
        const error = this.errors.get(errorId);
        if (error == null) {
            throw new Error(`Error ${error} does not exist`);
        }
        return error;
    }
}
